import java.io.*;
import java.net.*;
import java.util.*;


/**
 * Class to represent a job running on the server that generally performs
 * a specific task and then terminates.
 * @lastModified Mon Aug 31 15:24:35 PDT 1998
 */
public class SalesReportSessionBean {

	/*
	 * Jagg instance for database access.
	 */
	JDPJagg jaggSQL;
	/*
	 * Handle to the subsystem manager of the server.
	 */
	JDPSubsystemMgr manager;
	/*
	 * array to hold the report data from the database.
	 */
	String[][] reportData;
	/*
	 * Handle to the job that this class is running in.
	 */
	JDPJob thisJob;
	/*
	 * Handle to the utilities class.
	 */
	JDPUtils utils;

	/**
	 * Creates a new module that runs on the server, performs a task or 
	 * set of tasks and then terminates.
	 * @param manager the handle to the subsystem manager.
	 * @param thisJob the handle to the job in which this class runs.
	 * @param mailServer the machine name of the mail server (e.g. mail.mydomain.com).
	 * @param toAddress the email address of the recipient.
	 * @param returnAddress the email address of the sender.
	 */
	public SalesReportSessionBean(JDPSubsystemMgr manager, JDPJob thisJob, String mailServer, String toAddress, String returnAddress) {

		this.manager = manager;
		this.thisJob = thisJob;
		thisJob.setInstance(this);
		jaggSQL = new JDPJagg(manager.JDPJaggPath);
		utils = new JDPUtils(null);
		utils.jaggSQL = jaggSQL;

		thisJob.appendJobLog("Job started on " + (new Date()).toString());

		try {
			//
			// Insert tasks here
			//
			if (retrieveReportData() > 0) {
				String message = constructReport();
				sendEmail(mailServer, toAddress, returnAddress, message, "Sales Report");
			} else {
				thisJob.appendJobLog("Failed to retrieve any reportable data.");
			}	
		} catch (Exception e) {
			thisJob.logException(e);
		}

		thisJob.appendJobLog("Job completed on " + (new Date()).toString());

	}



	/**
	 * contruct the report from the report data.
	 * @return the constructed report.
	 */
	public String constructReport() {
		
		StringBuffer report = new StringBuffer();
		
		report.append("SALES REPORT");
		report.append("\n\n");
		report.append(utils.padr("Store",25));
		report.append(utils.padr("Date",12));
		report.append(utils.padr("Order Num",15));
		report.append(utils.padr("Title ID",12));
		report.append(utils.padr("Quantity",10));
		report.append("\n");
		report.append(utils.padr("=====",25));
		report.append(utils.padr("====",12));
		report.append(utils.padr("=========",15));
		report.append(utils.padr("========",12));
		report.append(utils.padr("========",10));
		report.append("\n");
		report.append("\n");
		
		String storeName = "";
		String orderDate = "";
		String orderNum = "";
		String titleId = "";
		String qty = "";
		String lastStoreName = "";
		String lastOrderDate = "";
		String lastOrderNum = "";
		int subtotal = 0;
		int grandtotal = 0;
		
		for (int ix=0; ix<reportData.length; ix++) {
			storeName = reportData[ix][4];
			orderDate = reportData[ix][1];
			orderNum = reportData[ix][0];
			titleId = reportData[ix][3];
			qty = reportData[ix][2];			
			if (!lastStoreName.equals(storeName)) {
				if (!lastStoreName.equals("")) {
					report.append(utils.padr("",25+12+15+12));
					report.append(utils.padl(Integer.toString(subtotal),8));
					report.append("\n\n");
					subtotal = 0;
				}
				report.append(utils.padr(storeName,25));
			} else {
				report.append(utils.padr("",25));
			}
			if (!lastStoreName.equals(storeName) || 
				!lastOrderDate.equals(orderDate)) {
				report.append(utils.padr(JDPDate.format(Long.valueOf(orderDate).longValue()),12));
			} else {
				report.append(utils.padr("",12));
			}
			if (!lastStoreName.equals(storeName) ||
				!lastOrderNum.equals(orderNum)) {
				report.append(utils.padr(orderNum,15));
			} else {
				report.append(utils.padr("",15));
			}
			report.append(utils.padr(titleId,12));
			report.append(utils.padl(qty,8));
			report.append("\n");
			
			subtotal += Integer.parseInt(qty);
			grandtotal += Integer.parseInt(qty);
										
			lastStoreName = storeName;
			lastOrderDate = orderDate;
			lastOrderNum = orderNum;
		}
		report.append(utils.padr("",25+12+15+12));
		report.append(utils.padl(Integer.toString(subtotal),8));
		report.append("\n\n");
		report.append(utils.padr("",25+12+15+12));
		report.append(utils.padl(Integer.toString(grandtotal),8));
		report.append("\n\n");

		return report.toString();	
	}


	/**
	 * Run an SQL to perform a Select to retrieve the report data from the
	 * database.
	 * @return the result code from the SQL call.
	 */
	public int retrieveReportData() {

		jaggSQL.loadJdbcSettings("JDP Tutorial");
		jaggSQL.setUID("");
		jaggSQL.setPWD("");
		jaggSQL.setMRW("1000");
		jaggSQL.setTOUT("60");

		Vector results = new Vector();
		String SQL = "SELECT a.ord_num, " +
					"a.ord_date, " +
					"a.qty, " +
					"a.title_id, " +
					"b.stor_name " +
				"FROM dbo_sales a, dbo_stores b " +
				"WHERE a.stor_id = b.stor_id ";
		int returnVal = jaggSQL.execSQL(SQL, results);

		reportData = new String[results.size()][5];
		StringTokenizer stok;
		for (int ix=0; ix<results.size(); ix++) {
			stok = new StringTokenizer((String)results.elementAt(ix),jaggSQL.getSEP());
			reportData[ix][0] = stok.nextToken();
			reportData[ix][1] = stok.nextToken();
			reportData[ix][2] = stok.nextToken();
			reportData[ix][3] = stok.nextToken();
			reportData[ix][4] = stok.nextToken();
		}
		return returnVal;

	}


	/**
	 * send the report via email to the requested recipient.
	 * @param mailServer name of the mailserver (e.g. mail.mydomain.com).
	 * @param toAddress the email address of the recipient.
	 * @param returnAddress the email address of the sender.
	 * @param message the message text.
	 * @param subject the message subject.
	 */
	public void sendEmail(String mailServer, String toAddress, String returnAddress, String message, String subject) {
		
		JDPSendMail mail = new JDPSendMail(mailServer);
		
		if (mail.mailMessage(toAddress, returnAddress, subject, message, "")) {
			thisJob.appendJobLog("Report successfully sent to " + toAddress + ".");
		} else {
			thisJob.appendJobLog("Failed to send report to " + toAddress + ".");
		}
		mail.closePort();
		
	}


}
